import json
from OpenHub.hardware_interfaces.channels.json.channel_encoder import ChannelEncoder

from OpenHub.hardware_interfaces.channels.channel_interface import ChannelInterface


class HomekitEncoder(json.JSONEncoder):

    def default(self, o):
        if isinstance(o, ChannelInterface):
            dict = o.__dict__
            dict['type'] = o.__name__
            dict['channel'] = json.dump(o.channel, cls= ChannelEncoder)
            #
            # if isinstance(o, DHT22Humidity):
            #     dict['type'] = 'DHT22Humidity'
            #     return dict
            # elif isinstance(o, DHT22Temp):
            #     return {}
            #
            # elif isinstance(o, MCP3008Analog):
            #     return {}
            #
            # elif isinstance(o, ModProbeTemp):
            #     return {}
            #
            # elif isinstance(o, PiPicoAnalog):
            #     return {}
            #
            # elif isinstance(o, PiPicoPump):
            #     return {}
            #
            # elif isinstance(o, VEML7700Light):
            #     return {}
            #
            # elif isinstance(o, VEML7700Lux):
            #     return {}

        return json.JSONEncoder.default(self, o)



from .channel_interface import ChannelInterface
import time
import logging
from OpenHub.globals import id_hardware_map


class DHT22Humidity(ChannelInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, config, hardware_serial_no=None, serial_no=None, dht_device=None,channel_stats=None, *args,
                 **kwargs):
        self.dht = id_hardware_map[config['hardware']]
        self.serial_no = config['id']
        self.type = config['type']

        super().__init__(config=config,hardware_serial_no=self.dht.serial_no, serial_no=self.serial_no, channel_stats=channel_stats,*args, **kwargs)

    async def get_raw_data(self):
        return {'value':float(self.dht.get_humidity())}



from abc import ABC, abstractmethod
import uuid
from OpenHub.globals import id_hardware_map


class HardwareInterface(ABC):

    def __init__(self, serial_no=uuid.uuid4(), *args, **kwargs):
        self.serial_no = serial_no

        id_hardware_map[str(self.serial_no)] = self






from .stat_interface import StatInterface

class Max(StatInterface):
    def __init__(self,dct=None,id=None,value=None,channel_serial=None):
        super().__init__(dct=dct, id=id, value=value, type='MAX',channel_serial=channel_serial)


    def update(self, value):
        if float(value) > self.value:
            self.value = float(value)
            self.check_if_value_updated()



import logging
import board
import busio

from hardware_interfaces.base_io.base_io_interface import BaseIOInterface


class I2C(BaseIOInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, scl=board.SCL, sda=board.SDA, *args, **kwargs):
        self.i2c = busio.I2C(scl, sda)
        super(I2C, self).__init__(*args, **kwargs)



from .channel_interface import ChannelInterface
import logging
from OpenHub.globals import id_hardware_map


class VEML7700Light(ChannelInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, config, hardware_serial_no=None, serial_no=None,channel_stats=None, *args, **kwargs):
        self.veml = id_hardware_map[config['hardware']]
        self.serial_no = config['id']
        self.type = config['type']
        super().__init__(config=config,hardware_serial_no=self.veml.serial_no, serial_no=self.serial_no,channel_stats=channel_stats, *args, **kwargs)

    async def get_raw_data(self):
        return {'value':self.veml.get_light()}



import json
import logging
import RPi.GPIO as GPIO
from serial import Serial
from .channels.pi_pico_pump import PiPicoPump
from .channels.pi_pico_analog import PiPicoAnalog
from .channels.pi_pico_relay import PiPicoRelay
from .hardware_interface import HardwareInterface

logger = logging.getLogger(__name__)
from asyncio import Lock


class PiPico(HardwareInterface):
    lock = Lock()

    serial_no = None
    name = None

    serial_com = None
    serial = None
    interrupt = None

    channels = []

    def __init__(self, pico_config, channels=None, *args, **kwargs):
        self.config = pico_config

        # self.channels_config = pico_config['channels']
        self.serial_no = pico_config['id']
        self.interrupt = pico_config['pi_gpio_interrupt']
        # if 'serial_com' in pico_config.keys():
        #     self.serial_com = pico_config['serial_com']
        # if self.serial_com is not None:
        #     self.serial = Serial(self.serial_com, 9600, timeout=1)
        GPIO.setup(self.interrupt, GPIO.OUT)
        super(PiPico, self).__init__(self.serial_no, channels, *args, **kwargs)

    def set_serial_com(self, serial_com):
        self.serial = serial_com

    def create_channel(self):
        for channel_config in self.channels_config.values():
            if 'type' == 'pump':
                self.channels.append(PiPicoPump(self, ))
            elif 'type' == 'relay':
                self.channels.append(PiPicoRelay(self, ))

    async def send_command(self, command, channel=None, state=None):
        response = await self.send_command_lock(command, channel, state, self.lock)
        return response

    async def send_command_lock(self, command, channel=None, state = None, lock=None):
        command_json = self.build_command_json(command,channel,state)
        sensor_response = b'{}'
        await lock.acquire()
        try:
            self.serial.flush()
            self.serial.write((json.dumps(command_json) + '\n').encode('utf-8'))
            pico_data = self.serial.readline()
            sensor_response = pico_data[:-2]
            logger.info(sensor_response)
            self.serial.flush()
        except Exception as error:
            logger.error(str(error.with_traceback()))
        finally:
            lock.release()
            return sensor_response

    def build_command_json(self, command, channel,state):
        command_json = {'command': command}
        if channel is not None:
            command_json['channel'] = channel
        if state is not None:
            command_json['state'] = state
        return command_json

    async def initialize(self):
        command = 'init'
        return await self.send_command(command)



import logging
from OpenHub.hardware_interfaces import hardware_interface
from OpenHub.hardware_interfaces.channels.dht22_humidity import DHT22Humidity
from OpenHub.hardware_interfaces.channels.dht22_temp import DHT22Temp
import time
from pigpio_dht import DHT22 as dht_22


class DHT22(hardware_interface.HardwareInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, pin_no=18, serial_no=None, channels=None, *args, **kwargs):
        self.dhtDevice = dht_22(pin_no)
        self.type = __name__
        super().__init__(serial_no, channels, *args, **kwargs)

    def create_channel(self):
        self.create_channel_temp()
        self.create_channel_humidity()

    def create_channel_temp(self):
        self.logger.info('Creating DHT22 Temp Channel: ')
        self.channels["temp"] = DHT22Temp(hardware_serial_no=self.serial_no, dht_device=self.dhtDevice)

    def create_channel_humidity(self):
        self.logger.info('Creating DHT22 Humidity Channel: ')
        self.channels["humidity"] = DHT22Humidity(hardware_serial_no=self.serial_no, dht_device=self.dhtDevice)

    def get_channel(self, channel):
        return self.channels[channel]

    def get_data(self, type):
        valid = False
        while valid is False:
            data = self.dhtDevice.read()
            valid = data['valid']
        return data[type]

    def get_temp_c(self):
        return self.get_data('temp_c')

    def get_temp_f(self):
        return self.get_data('temp_f')

    def get_humidity(self):
        return self.get_data('humidity')



import socket
import subprocess

# Start a socket listening for connections on 0.0.0.0:8000 (0.0.0.0 means
# all interfaces)
server_socket = socket.socket()
server_socket.bind(('0.0.0.0', 8000))
server_socket.listen(0)

# Accept a single connection and make a file-like object out of it
connection = server_socket.accept()[0].makefile('rb')
try:
    # Run a viewer with an appropriate command line. Uncomment the mplayer
    # version if you would prefer to use mplayer instead of VLC
    cmdline = ['vlc', '--demux', 'h264', '-']
    #cmdline = ['mplayer', '-fps', '25', '-cache', '1024', '-']
    player = subprocess.Popen(cmdline, stdin=subprocess.PIPE)
    while True:
        # Repeatedly read 1k of data from the connection and write it to
        # the media player's stdin
        data = connection.read(1024)
        if not data:
            break
        player.stdin.write(data)
finally:
    connection.close()
    server_socket.close()
    player.terminate()




from abc import ABC, abstractmethod
import uuid

class StatInterface(ABC):
    def __init__(self, dct, id=None, value=None,type=type,channel_serial=None):
        if dct is not None:
            if 'id' in dct:
                self.id = dct['id']
            else:
                self.id = str(uuid.uuid4())
            if 'value' in dct:
                self.old_value = dct['value']
                self.value = dct['value']
            if 'channel' in dct:
                self.channel_serial = dct['channel']
        else:
            self.id = id
            self.value = value
            self.old_value = value
            self.type = type
            self.channel_serial = channel_serial
        self.update_on_server = False

    @abstractmethod
    def update(self, value):
        pass

    def check_if_value_updated(self):
        if self.old_value != self.value:
            self.update_on_server = True

    def value_updated_on_server(self):
        self.old_value = self.value
        self.update_on_server = False;


import time
import board
import busio
from digitalio import DigitalInOut, Direction, Pull
from adafruit_pm25.i2c import PM25_I2C




import logging
from OpenHub.hardware_interfaces import hardware_interface
from OpenHub.hardware_interfaces.channels.mcp3008analog import MCP3008Analog


class MCP3008(hardware_interface.HardwareInterface):
    logger = logging.getLogger(__name__)

    reset_pin = None

    # If you have a GPIO, its not a bad idea to connect it to the RESET pin
    # reset_pin = DigitalInOut(board.G0)
    # reset_pin.direction = Direction.OUTPUT
    # reset_pin.value = False

    def __init__(self, scl=board.SCL, sda=board.SDA, serial_no=None, channels=None, reset_pin_num=board.G0, *args, **kwargs):
        reset_pin = DigitalInOut(reset_pin_num)
        reset_pin.direction = Direction.OUTPUT
        reset_pin.value = False
        i2c = busio.I2C(scl, sda, frequency=100000)
        # Connect to a PM2.5 sensor over I2C
        pm25 = PM25_I2C(i2c, reset_pin)
        self.type = __name__
        super().__init__(serial_no, channels, *args, **kwargs)

    def create_channel(self):
        pin = str(input("Which channel is this (0-8)?"))
        return self.create_analog_channel(pin)





from pyhap.const import CATEGORY_SENSOR
import logging
import OpenHub.hardware_interfaces.veml_7700 as veml
from OpenHub.homekit_accessories.homkit_sensor_interface import HomeKitSensorInterface


class LightSensor(HomeKitSensorInterface):
    logger = logging.getLogger(__name__)
    run_debug_message = "Current Light Level: "


    index = None
    channel = None
    light_service = None
    light_service2 = None
    char_ambient = None
    char_lux = None
    category = CATEGORY_SENSOR
    serial_no = None
    name = None
    display_name = None

    def __init__(self, serial_no=None, display_name=None, channel_interface_serial_no=None, config=None, *args, **kwargs):
        if self.display_name is None:
            self.display_name = display_name + "LightSensor"
        super().__init__(serial_no=serial_no, display_name=display_name,
                         channel_interface_serial_no=channel_interface_serial_no,config=config, *args, **kwargs)

    def set_display_name(self, display_name):
        if display_name is None:
            return "Light Sensor"
        else:
            return display_name


    def add_functional_service(self):
        return self.add_preload_service("LightSensor")

    def add_functional_service_characteristic(self):
        return self.service.get_characteristic('CurrentAmbientLightLevel')




from abc import ABC, abstractmethod
import uuid
from .stats.stats import Stats
from OpenHub.hardware_interfaces.channels.stat.max import Max
from OpenHub.hardware_interfaces.channels.stat.min import Min

class ChannelInterface(ABC):
    def __init__(self, config, hardware_serial_no=None, serial_no=uuid.uuid4(), channel_stats=None, **kwargs):
        self.serial_no = serial_no
        self.hardware_serial_no = hardware_serial_no

        if 'keep_statistics' in config:
            self.keep_statistics = config['keep_statistics']
        else:
            self.keep_statistics = False

        if self.keep_statistics:
            if channel_stats is not None and len(channel_stats>0):
                self.stats = Stats(channel_stats)
            else:
                min = Min(id=str(uuid.uuid4()),value=float('inf'),channel_serial=self.serial_no)
                max = Max(id=str(uuid.uuid4()),value=float('-inf'),channel_serial=self.serial_no)
                self.stats = Stats([min,max])

    def update_stats(self,value):
        if self.keep_statistics:
            self.stats.update(value)
            self.stats.update_on_server()

    async def run(self):
        value = await self.get_raw_data()
        if self.keep_statistics:
            if 'averaged' in value.keys():
                self.update_stats(value['averaged'])
            elif 'value' in value.keys():
                self.update_stats(value['value'])
            else:
                self.update_stats(value)
        return value


    @abstractmethod
    async def get_raw_data(self):
        pass



import logging
from .data_transformer import DataTransformer


class Inverse(DataTransformer):

    def perform_op(self, inputs):
        for input in inputs:
            product = float(1/float(input))
        return product


from .stat_interface import StatInterface

class Min(StatInterface):
    def __init__(self,dct=None,id=None,value=None,channel_serial=None):
        super().__init__(dct=dct, id=id, value=value, type='MIN',channel_serial=channel_serial)

    def update(self, value):
        if float(value) < self.value:
            self.value = float(value)
            self.check_if_value_updated()



import json
from data_transformers.divide import Divide
from data_transformers.max import Max
from data_transformers.min import Min
from data_transformers.product import Product
from data_transformers.difference import Difference
from data_transformers.sum import Sum

class DataTransformerDecoder(json.JSONDecoder):

    def __init__(self, *args, **kwargs):
        json.JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)

    def object_hook(self, dct):
        data_transformers = []
        if 'data_transformers' in dct and dct['data_transformers'] is not None:
            for data_transformer in dct['data_transformers']:
                transformer = json.loads(data_transformer, cls=self.__class__)
                data_transformers.append(transformer)

        type = dct['type']

        if type == 'Divide':
            return Divide(dct,data_transformers)
        if type == 'Max':
            return Max(dct,data_transformers)
        if type == 'Multiply':
            return Product(dct,data_transformers)
        if type == 'Subtract':
            return Difference(dct,data_transformers)
        if type == 'Min':
            return Min(dct,data_transformers)
        if type == 'Sum':
            return Sum(dct,data_transformers)



from abc import ABC, abstractmethod
from pyhap.accessory import Accessory
from pyhap.accessory import CATEGORY_OTHER
import logging
from OpenHub.globals import id_channels_map \
    , driver, accessory_id_data_transformer_map
import uuid


class HomeKitSensorInterface(ABC, Accessory):
    logger = logging.getLogger(__name__)
    category = CATEGORY_OTHER
    scale = None
    index = None
    service = None
    char = None

    serial_no = None
    display_name = None

    channel = None

    raw_value_converter = None

    run_debug_message = "Run Debug Message Not Implemented"

    calibrator = None
    aid = None

    data_transformer = None

    def __init__(self, serial_no=uuid.uuid4(), display_name=None, channel_interface_serial_no=None,
                 data_transformer=None,config=None, *args, **kwargs):
        ABC.__init__(self)
        self.display_name = self.set_display_name(display_name)
        self.serial_no = serial_no
        self.channel_serial_no = channel_interface_serial_no
        self.channel = id_channels_map[str(self.channel_serial_no)]
        Accessory.__init__(self, driver=driver, display_name=self.display_name,
                           *args, **kwargs)

        self.service = self.add_functional_service()
        self.char = self.add_functional_service_characteristic()
        if config is not None and 'datatransformer' in config.keys():
            self.data_transformer = config['datatransformer']
        if config is not None and 'data_transformer' in config.keys():
            self.data_transformer = config['data_transformer']


    def add_info_service(self):
        serv_info = self.driver.loader.get_service("AccessoryInformation")
        serv_info.configure_char("Name", value=self.display_name)
        serv_info.configure_char("SerialNumber", value=self.serial_no)
        serv_info.configure_char("Manufacturer", value="BellyFrito")
        serv_info.configure_char("Model", value="DEFAULT")
        self.add_service(serv_info)

    @abstractmethod
    def set_display_name(self, display_name):
        pass

    @abstractmethod
    def add_functional_service_characteristic(self):
        pass

    @abstractmethod
    def add_functional_service(self):
        pass

    async def run(self):
        if self.data_transformer is None:
            data = await self.channel.run()
            self.logger.info(self.display_name + " Output: " + str(data))
            if 'averaged' in data.keys():
                if self.scale is None:
                    self.char.set_value(float(data['averaged']))
                else:
                    self.char.set_value(self.scale*float(data['averaged']))
            elif 'value' in data.keys():
                if self.scale is None:
                    self.char.set_value(float(data['value']))
                else:
                    self.char.set_value(self.scale * float(data['value']))
        else:
            val = await self.data_transformer.run()
            self.logger.info(self.display_name + " Output: " + str(val))

            self.char.set_value(float(val))


import os
import socket
import multiprocessing
import subprocess
import os
import logging

logger = logging.getLogger(__name__)


def pinger(job_q, results_q):
    """
    Do Ping
    :param job_q:
    :param results_q:
    :return:
    """
    while True:

        ip = job_q.get()

        if ip is None:
            break

        try:
            logger.debug('Pinging: ' + str(ip))
            subprocess.check_call(['/usr/bin/ping', '-c', '1', '-W', '0.5', '-w', '0.5', str(ip)], timeout=0.5,
                                  shell=False, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            logger.debug('added: ' + str(ip))
            results_q.put(ip)
        except Exception as e:
            logger.debug(str(e))
            logger.debug(str(e.with_traceback(e.__traceback__)))
            pass


def get_my_ip():
    """
    Find my IP address
    :return:
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect(("8.8.8.8", 80))
    ip = s.getsockname()[0]
    s.close()
    logger.debug('My IP: ' + str(ip))

    return ip


def map_network(pool_size=4):
    """
    Maps the network
    :param pool_size: amount of parallel ping processes
    :return: list of valid ip addresses
    """

    ip_list = list()

    # get my IP and compose a base like 192.168.1.xxx
    ip_parts = get_my_ip().split('.')
    base_ip = ip_parts[0] + '.' + ip_parts[1] + '.' + ip_parts[2] + '.'

    # prepare the jobs queue
    jobs = multiprocessing.Queue()
    results = multiprocessing.Queue()

    pool = [multiprocessing.Process(target=pinger, args=(jobs, results)) for i in range(pool_size)]

    for p in pool:
        p.start()

    # cue hte ping processes
    for i in range(1, 255):
        jobs.put(base_ip + '{0}'.format(i))

    for p in pool:
        jobs.put(None)

    for p in pool:
        p.join()

    # collect he results
    while not results.empty():
        ip = results.get()
        ip_list.append(ip)

    return ip_list


import socket


def get_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        # doesn't even have to be reachable
        s.connect(('10.255.255.255', 1))
        IP = s.getsockname()[0]
    except Exception:
        IP = '127.0.0.1'
    finally:
        s.close()
        logger.debug('My IP: ' + str(IP))

    return IP


def non_multiprocessing():
    ip_parts = get_my_ip().split('.')
    base_ip = ip_parts[0] + '.' + ip_parts[1] + '.' + ip_parts[2] + '.'

    ips = []
    for i in range(1, 255):
        ip = base_ip + '{0}'.format(i)
        print(str(ip))

        try:
            subprocess.check_call(['/usr/bin/ping', '-c', '1', '-W', '0.3', '-w', '0.3', str(ip)], timeout=0.3,
                                  shell=False, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            logger.debug('adding ' + str(ip))
            ips.append(ip)
        except Exception as a:
            logger.debug(a)
    logger.debug(str(ips))
    return ips


if __name__ == '__main__':
    print('Mapping...')
    lst = map_network()
    print(lst)
